--[[
Simulation metamodel

]]--
tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/courselib/?.lua;"

local view = require ("viewer")
local scenario = {}

__ScenarioModel = {
    
    createGraphics = function (self)
        -- create the plot to visualise the results
        
        local atts = {}
        for _, name in pairs (self.outcomes) do
            table.insert (atts, name) 
        end
        self.plot = view.Plot {
            subject    = Cell (self.outcomes),
            attributes = atts
        }
    end,
    
    createTimer = function (self)
    	local timer = Timer {
		Event {time = 1, period = 1, action = 
                function(ev)
                    self:changes()     --
                    self.plot:notify(ev)
                end
               }
        }
        return timer
    end,
    
    run  = function (self, ntimes)
                self.timer:execute (ntimes)
           end
}

-- defines the metatable for the System Dynamics Model
metaTableScenario = {__index = __ScenarioModel}


function scenario.Model (argv)

    -- first gets the user-defined parameters
	local model = argv
    -- at least subject and attribute must be provided
    assert (model.policies  ~= nil, "Scenario: policies must be provided")
    assert (model.outcomes  ~= nil, "Scenario: outcomes must be provided") 

	-- all other parameters are accessed via the metatable
	setmetatable(model, metaTableScenario)
    
    -- create the views for the map
    model:createGraphics()
    
    -- create the timer for the model
    model.timer  = model:createTimer()
    return model
 end
 
 return scenario
    