-- Simple statistical functions              --
----------------------------------------------
-- Table to hold statistical functions

local stats={}

-- Get the mean value of a table of values
function stats.mean( tab )
  local sum   = 0
  local count = 0
  for k,v in pairs(tab) do
    if type(v) == 'number' then
      sum = sum + v
      count = count + 1
    end
  end
  return (sum / count)
end

-- Get the mode of a table.  
-- Returns a table of values.
-- Works on anything (not just numbers).

function stats.mode (tab)
    local counts={}
    for k,v in pairs (tab) do
        if counts[v] == nil then
            counts[v] = 1
        else
            counts[v] = counts[v] + 1
        end
    end

    local biggestCount = 0
    for k,v in pairs (counts) do
        if v > biggestCount then
            biggestCount = v
        end
    end

    local temp={}
    for k,v in pairs( counts ) do
        if v == biggestCount then
            table.insert( temp, k )
        end
    end
    return temp
end

-- Get the median of a table.
function stats.median (tab)
    table.sort (tab)
    mid = #tab/2
    -- If we have an even number of table elements or odd.
    if #tab % 2 == 0 then
    -- return mean value of middle two elements
        return (tab[mid] + tab[mid + 1])/2
    else
        -- return middle element
        return tab[math.ceil(mid)]
    end
end
    

-- Get the standard deviation of a table
function stats.stdev( tab )
  local vm  = 0
  local sum = 0
  local count = 0

  local m = stats.mean(tab)

  for k,v in pairs(tab) do
    if type(tab) == 'number' then
      vm = v - m
      sum = sum + (vm * vm)
      count = count + 1
    end
  end

  return math.sqrt(sum /(count-1))
end

-- Get the max and min for a table
function stats.minmax( tab )
  local max = -math.huge
  local min = math.huge

  for k,v in pairs( tab ) do
    if type(v) == 'number' then
      max = math.max( max, v )
      min = math.min( min, v )
    end
  end

  return min, max
end

--[[
  Calculate a cumulative distribution function 
]]
function stats.distrib (tab)
    assert (#t >= 1, "distrib: number of entries should be positive")
    table.sort (tab) 

    min, max = stats.minmax (tab)
    assert (min >= 1, "distribution: minimum value should be greater than 1")

    local dist = {}
    for e = 1, max do
        dist[e] = ms.count (t, e) 
    end

return dist
end 

return stats