-- Support for System Dynamics Model

--- Configuration variable for contributed TerraME libraries
--  @usage tmepath should point to the TerraME installation directory
tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/courselib/?.lua;"

local laux = require ("laux")
local show = require ("show")

local sysdyn = {}

__SystemDynamicsModel = {
    
    delta_time  = 1,  -- integration step
    update_time = 1,  -- visualisation step
    
    createGraphics = function (self)
        show.createGraphics(self)
    end,
    
    updateGraphics = function (self, ev)
        show.updateGraphics (self, ev)
    end,
    
    createTimer = function (self)
    	local timer = Timer {
		Event {time = 1, period = self.delta_time, action = 
                function(ev)
                    self:changes()  
                end
               },
        Event {time = 1, period = self.update_time, action = 
                function(ev)  
                    self:updateGraphics(ev)
                end
               },
        Event {time = 1, period = 1, action = 
                function(ev)  
                    self.event_time = ev:getTime()
                end
               }
        }
        return timer
    end,
    
    run  = function (self, ntimes)
                self.maxtimes = ntimes
                self.timer:execute (ntimes)
           end
}

-- defines the metatable for the System Dynamics Model
metaTableSysDynModel = {__index = __SystemDynamicsModel}

function sysdyn.Model(argv)
    -- first gets the user-defined parameters
	local model = argv
    -- at least subject and attribute must be provided
    assert (model.graphics  ~= nil, "System Dynamics: graphics specs must be provided")
    assert (model.changes  ~= nil, "System Dynamics: model changes must be provided") 

	-- all other parameters are accessed via the metatable
	setmetatable(model, metaTableSysDynModel)
    
    -- create the views for the map
    model:createGraphics()
    
    -- create the timer for the model
    model.timer  = model:createTimer()
    
    return model
end

function sysdyn.diff (argv)
    assert (argv.equations ~= nil, "DiffEquation: equations must be provided")
    assert (argv.xs ~= nil, "DiffEquation: initial values must be provided")
    
    return  integrate {
        equation = argv.equations, 
        method = "euler",
        initial = argv.xs,
        a = 0.0,
        b = 1.0,
        step = 0.03125 
    }
end

return sysdyn